@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2000, 2003, 2004, 2005, 2006, 2007, 2009 by              */
+/*  Copyright 1996-2000, 2003-2007, 2009-2011 by                            */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -40,24 +40,35 @@
 #endif
 
 
-/* these variables, structures and declarations  are for  */
-/* communication with the debugger in the autofit module; */
-/* normal programs don't need this                        */
-struct  AF_GlyphHintsRec_;
-typedef struct AF_GlyphHintsRec_*  AF_GlyphHints;
+  /* these variables, structures and declarations  are for  */
+  /* communication with the debugger in the autofit module; */
+  /* normal programs don't need this                        */
+  struct  AF_GlyphHintsRec_;
+  typedef struct AF_GlyphHintsRec_*  AF_GlyphHints;
 
-int            _af_debug;
-int            _af_debug_disable_horz_hints;
-int            _af_debug_disable_vert_hints;
-int            _af_debug_disable_blue_hints;
-AF_GlyphHints  _af_debug_hints;
+  int            _af_debug_disable_horz_hints;
+  int            _af_debug_disable_vert_hints;
+  int            _af_debug_disable_blue_hints;
+  AF_GlyphHints  _af_debug_hints;
 
 #ifdef __cplusplus
   extern "C" {
 #endif
-extern void af_glyph_hints_dump_segments( AF_GlyphHints  hints );
-extern void af_glyph_hints_dump_points( AF_GlyphHints  hints );
-extern void af_glyph_hints_dump_edges( AF_GlyphHints  hints );
+  extern void
+  af_glyph_hints_dump_segments( AF_GlyphHints  hints );
+  extern void
+  af_glyph_hints_dump_points( AF_GlyphHints  hints );
+  extern void
+  af_glyph_hints_dump_edges( AF_GlyphHints  hints );
+  extern FT_Error
+  af_glyph_hints_get_num_segments( AF_GlyphHints  hints,
+                                   FT_Int         dimension,
+                                   FT_Int*        num_segments );
+  extern FT_Error
+  af_glyph_hints_get_segment_offset( AF_GlyphHints  hints,
+                                     FT_Int         dimension,
+                                     FT_Int         idx,
+                                     FT_Pos*        offset );
 #ifdef __cplusplus
   }
 #endif
@@ -88,12 +99,14 @@ typedef struct  status_
   grColor      on_color;
   grColor      conic_color;
   grColor      cubic_color;
+  grColor      segment_color;
 
   int          do_horz_hints;
   int          do_vert_hints;
   int          do_blue_hints;
   int          do_outline;
   int          do_dots;
+  int          do_segment;
 
   double       gamma;
   const char*  header;
@@ -117,6 +130,7 @@ grid_status_init( GridStatus       st,
   st->on_color      = grFindColor( display->bitmap,  64,  64, 255, 255 );
   st->conic_color   = grFindColor( display->bitmap,   0, 128,   0, 255 );
   st->cubic_color   = grFindColor( display->bitmap, 255,  64, 255, 255 );
+  st->segment_color = grFindColor( display->bitmap,  64, 255, 128,  64 );
   st->disp_width    = display->bitmap->width;
   st->disp_height   = display->bitmap->rows;
   st->disp_bitmap   = display->bitmap;
@@ -126,6 +140,7 @@ grid_status_init( GridStatus       st,
   st->do_blue_hints = 1;
   st->do_dots       = 1;
   st->do_outline    = 1;
+  st->do_segment    = 0;
 
   st->Num    = 0;
   st->gamma  = 1.0;
@@ -220,6 +235,48 @@ grid_status_draw_grid( GridStatus  st )
 }
 
 
+static void
+grid_hint_draw_segment( GridStatus     st,
+                        AF_GlyphHints  hints )
+{
+  FT_Int  dimension;
+  int     x_org = (int)st->x_origin;
+  int     y_org = (int)st->y_origin;
+
+
+  for ( dimension = 1; dimension >= 0; dimension-- )
+  {
+    FT_Int  num_seg;
+    FT_Int  count;
+
+
+    af_glyph_hints_get_num_segments( hints, dimension, &num_seg );
+
+    for ( count = 0; count < num_seg; count++ )
+    {
+      int     pos;
+      FT_Pos  offset;
+
+
+      af_glyph_hints_get_segment_offset( hints, dimension, count, &offset );
+
+      if ( dimension == 0 ) /* AF_DIMENSION_HORZ is 0 */
+      {
+        pos = x_org + (int)offset * st->scale;
+        grFillVLine( st->disp_bitmap, pos, 0,
+                     st->disp_height, st->segment_color );
+      }
+      else
+      {
+        pos = y_org - (int)offset * st->scale;
+        grFillHLine( st->disp_bitmap, 0, pos,
+                     st->disp_width, st->segment_color );
+      }
+    }
+  }
+}
+
+
 static void
 ft_bitmap_draw( FT_Bitmap*       bitmap,
                 int              x,
@@ -397,11 +454,27 @@ grid_status_draw_outline( GridStatus       st,
 
   FTDemo_Get_Size( handle, &size );
 
+  /* Draw segment before drawing glyph. */
+  if ( status.do_segment )
+  {
+    /* Force hinting first in order to collect segment info. */
+    _af_debug_disable_horz_hints = 0;
+    _af_debug_disable_vert_hints = 0;
+
+    if ( !FT_Load_Glyph( size->face, st->Num,
+                         FT_LOAD_DEFAULT        |
+                         FT_LOAD_NO_BITMAP      |
+                         FT_LOAD_FORCE_AUTOHINT |
+                         FT_LOAD_TARGET_NORMAL ) )
+      grid_hint_draw_segment( &status, _af_debug_hints );
+  }
+
   _af_debug_disable_horz_hints = !st->do_horz_hints;
   _af_debug_disable_vert_hints = !st->do_vert_hints;
 
-  FT_Load_Glyph( size->face, st->Num,
-                 handle->load_flags | FT_LOAD_NO_BITMAP );
+  if ( FT_Load_Glyph( size->face, st->Num,
+                      handle->load_flags | FT_LOAD_NO_BITMAP ) )
+    return;
 
   slot = size->face->glyph;
   if ( slot->format == FT_GLYPH_FORMAT_OUTLINE )
@@ -500,6 +573,8 @@ grid_status_draw_outline( GridStatus       st,
     grWriteln( "  F1 or ?    : display this help screen" );
     grLn();
     grWriteln( "  a          : toggle anti-aliasing" );
+    grWriteln( "  f          : toggle forced autofit mode" );
+    grWriteln( "  h          : toggle ignore hinting mode" );
     grWriteln( "  left/right : decrement/increment glyph index" );
     grWriteln( "  up/down    : change character size" );
     grLn();
@@ -517,9 +592,12 @@ grid_status_draw_outline( GridStatus       st,
     grWriteln( "  Page up/dn : zoom in/out grid" );
     grWriteln( "  RETURN     : reset zoom and position" );
     grLn();
+#ifdef FT_DEBUG_AUTOFIT
     grWriteln( "  H          : toggle horizontal hinting" );
     grWriteln( "  V          : toggle vertical hinting" );
     grWriteln( "  B          : toggle blue zone hinting" );
+    grWriteln( "  s          : toggle segment drawing" );
+#endif
     grWriteln( "  d          : toggle dots display" );
     grWriteln( "  o          : toggle outline display" );
     grWriteln( "  g          : increase gamma by 0.1" );
@@ -529,10 +607,12 @@ grid_status_draw_outline( GridStatus       st,
     grWriteln( "  p          : previous font" );
     grWriteln( "  q / ESC    : exit program" );
     grLn();
+#ifdef FT_DEBUG_AUTOFIT
     grWriteln( "  1          : dump edge hints" );
     grWriteln( "  2          : dump segment hints" );
     grWriteln( "  3          : dump point hints" );
     grLn();
+#endif
     grWriteln( "press any key to exit this help screen" );
 
     grRefreshSurface( display->surface );
@@ -580,8 +660,8 @@ grid_status_draw_outline( GridStatus       st,
   {
     status.scale *= zoom;
 
-    sprintf( status.header_buffer, "zoom level %.2f %%\n",
-             status.scale / status.scale_0 );
+    sprintf( status.header_buffer, "zoom level %.0f%%",
+             status.scale * 100.0 / status.scale_0 );
 
     status.header = (const char *)status.header_buffer;
   }
@@ -667,17 +747,46 @@ grid_status_draw_outline( GridStatus       st,
       FTDemo_Update_Current_Flags( handle );
       break;
 
+    case grKEY( 'f' ):
+      handle->autohint = !handle->autohint;
+      status.header    = handle->autohint ? "forced auto-hinting is now on"
+                                          : "forced auto-hinting is now off";
+
+      FTDemo_Update_Current_Flags( handle );
+      break;
+
+
+#ifdef FT_DEBUG_AUTOFIT
     case grKEY( '1' ):
-      af_glyph_hints_dump_edges( _af_debug_hints );
+      if (handle->hinted && handle->autohint)
+      {
+        status.header = "dumping glyph edges to stdout";
+        af_glyph_hints_dump_edges( _af_debug_hints );
+      }
+      else
+        status.header = "need autofit mode for edge dumping";
       break;
 
     case grKEY( '2' ):
-      af_glyph_hints_dump_segments( _af_debug_hints );
+      if (handle->hinted && handle->autohint)
+      {
+        status.header = "dumping glyph segments to stdout";
+        af_glyph_hints_dump_segments( _af_debug_hints );
+      }
+      else
+        status.header = "need autofit mode for segment dumping";
       break;
 
     case grKEY( '3' ):
-      af_glyph_hints_dump_points( _af_debug_hints );
+      if (handle->hinted && handle->autohint)
+      {
+        status.header = "dumping glyph points to stdout";
+        af_glyph_hints_dump_points( _af_debug_hints );
+      }
+      else
+        status.header = "need autofit mode for point dumping";
       break;
+#endif /* FT_DEBUG_AUTOFIT */
 
 
     case grKEY( 'g' ):
@@ -712,26 +821,48 @@ grid_status_draw_outline( GridStatus       st,
       event_font_change( -1 );
       break;
 
-    case grKEY('H'):
-      status.do_horz_hints = !status.do_horz_hints;
-      status.header = status.do_horz_hints ? "horizontal hinting enabled"
-                                           : "horizontal hinting disabled";
+
+#ifdef FT_DEBUG_AUTOFIT
+    case grKEY( 'H' ):
+      if (handle->autohint)
+      {
+        status.do_horz_hints = !status.do_horz_hints;
+        status.header = status.do_horz_hints ? "horizontal hinting enabled"
+                                             : "horizontal hinting disabled";
+      }
+      else
+        status.header = "need autofit mode to toggle horizontal hinting";
       break;
 
-    case grKEY('V'):
-      status.do_vert_hints = !status.do_vert_hints;
-      status.header        = status.do_vert_hints
-                             ? "vertical hinting enabled"
-                             : "vertical hinting disabled";
+    case grKEY( 'V' ):
+      if (handle->autohint)
+      {
+        status.do_vert_hints = !status.do_vert_hints;
+        status.header = status.do_vert_hints ? "vertical hinting enabled"
+                                             : "vertical hinting disabled";
+      }
+      else
+        status.header = "need autofit mode to toggle vertical hinting";
       break;
 
-    case grKEY('B'):
-      status.do_blue_hints = !status.do_blue_hints;
-      status.header        = status.do_blue_hints
-                             ? "blue zone hinting enabled"
-                             : "blue zone hinting disabled";
+    case grKEY( 'B' ):
+      if (handle->autohint)
+      {
+        status.do_blue_hints = !status.do_blue_hints;
+        status.header = status.do_blue_hints ? "blue zone hinting enabled"
+                                             : "blue zone hinting disabled";
+      }
+      else
+        status.header = "need autofit mode to toggle blue zone hinting";
       break;
 
+    case grKEY( 's' ):
+      status.do_segment = !status.do_segment;
+      status.header = status.do_segment ? "segment drawing enabled"
+                                        : "segment drawing disabled";
+      break;
+#endif /* FT_DEBUG_AUTOFIT */
+
 
     case grKeyLeft:     event_index_change( -1 ); break;
     case grKeyRight:    event_index_change( +1 ); break;
@@ -750,10 +881,10 @@ grid_status_draw_outline( GridStatus       st,
                         status.do_vert_hints = 1;
                         break;
 
-    case grKEY('i'):    event_grid_translate( 0, +1 ); break;
-    case grKEY('k'):    event_grid_translate( 0, -1 ); break;
-    case grKEY('l'):    event_grid_translate( +1, 0 ); break;
-    case grKEY('j'):    event_grid_translate( -1, 0 ); break;
+    case grKEY( 'i' ):  event_grid_translate( 0, +1 ); break;
+    case grKEY( 'k' ):  event_grid_translate( 0, -1 ); break;
+    case grKEY( 'l' ):  event_grid_translate( +1, 0 ); break;
+    case grKEY( 'j' ):  event_grid_translate( -1, 0 ); break;
 
     case grKeyPageUp:   event_grid_zoom( 1.25 ); break;
     case grKeyPageDown: event_grid_zoom( 1/1.25 ); break;
@@ -786,22 +917,22 @@ grid_status_draw_outline( GridStatus       st,
       switch ( error_code )
       {
       case FT_Err_Ok:
-        sprintf( status.header_buffer, "%s %s (file `%s')",
+        sprintf( status.header_buffer, "%.50s %.50s (file `%.100s')",
                  face->family_name, face->style_name, basename );
         break;
 
       case FT_Err_Invalid_Pixel_Size:
-        sprintf( status.header_buffer, "Invalid pixel size (file `%s')",
+        sprintf( status.header_buffer, "Invalid pixel size (file `%.100s')",
                  basename );
         break;
 
       case FT_Err_Invalid_PPem:
-        sprintf( status.header_buffer, "Invalid ppem value (file `%s')",
+        sprintf( status.header_buffer, "Invalid ppem value (file `%.100s')",
                  basename );
         break;
 
       default:
-        sprintf( status.header_buffer, "File `%s': error 0x%04x",
+        sprintf( status.header_buffer, "File `%.100s': error 0x%04x",
                  basename, (FT_UShort)error_code );
         break;
       }
@@ -850,16 +981,30 @@ grid_status_draw_outline( GridStatus       st,
   static void
   usage( char*  execname )
   {
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "ftgrid: simple glyph grid viewer -- part of the FreeType project\n" );
-    fprintf( stderr,  "-----------------------------------------------------------\n" );
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "Usage: %s [status below] ppem fontname[.ttf|.ttc] ...\n",
-             execname );
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "  -r R      use resolution R dpi (default: 72 dpi)\n" );
-    fprintf( stderr,  "  -f index  specify first index to display\n" );
-    fprintf( stderr,  "\n" );
+    fprintf( stderr,
+      "\n"
+      "ftgrid: simple glyph grid viewer -- part of the FreeType project\n"
+      "----------------------------------------------------------------\n"
+      "\n" );
+    fprintf( stderr,
+      "Usage: %s [options] pt font ...\n"
+      "\n", execname );
+    fprintf( stderr,
+      "  pt        The point size for the given resolution.\n"
+      "            If resolution is 72dpi, this directly gives the\n"
+      "            ppem value (pixels per EM).\n" );
+    fprintf( stderr,
+      "  font      The font file(s) to display; if the extension is omitted,\n"
+      "            ftview tries to append one of the following suffixes:\n"
+      "            `.ttf', `.ttc', `.otf', `.pfa', `.pfb'.\n"
+      "            For `.pfa' or `.pfb' files, ftview also tries to attach\n"
+      "            the corresponding metrics file (with extension\n"
+      "            `.afm' or `.pfm').\n"
+      "\n" );
+    fprintf( stderr,
+      "  -r R      Use resolution R dpi (default: 72dpi).\n"
+      "  -f index  Specify first index to display (default: 0).\n"
+      "\n" );
 
     exit( 1 );
   }
@@ -945,7 +1090,7 @@ grid_status_draw_outline( GridStatus       st,
     if ( handle->num_fonts == 0 )
       Fatal( "could not find/open any font file" );
 
-    printf( "ptsize =%g \n", status.ptsize/64.0 );
+    printf( "ptsize =%g\n", status.ptsize / 64.0 );
     FTDemo_Set_Current_Charsize( handle, status.ptsize, status.res );
     FTDemo_Update_Current_Flags( handle );
 
@@ -953,8 +1098,6 @@ grid_status_draw_outline( GridStatus       st,
 
     grid_status_rescale_initial( &status, handle );
 
-    _af_debug = 1;
-
     for ( ;; )
     {
       FTDemo_Display_Clear( display );
